@@ -5,7 +5,7 @@
 //
 
 /*
-Copyright or  or Copr. CNRS
+Copyright or  or Copr. Bio++ Development Team
 
 This software is a computer program whose purpose is to simulate sequence
 data according to a phylogenetic tree and an evolutionary model.
@@ -59,6 +59,10 @@ using namespace std;
 #include <Bpp/Seq/SequenceTools.h>
 #include <Bpp/Seq/GeneticCode.all>
 
+//From PhylLib:
+#include <Bpp/Phyl/Tree.h>
+#include <Bpp/Phyl/App/PhylogeneticsApplicationTools.h>
+
 using namespace bpp;
 
 void help()
@@ -74,8 +78,8 @@ void help()
 int main(int args, char** argv)
 {
   cout << "******************************************************************" << endl;
-  cout << "*           Bio++ Sequence Manipulator, version 0.4              *" << endl;
-  cout << "* Author: J. Dutheil                        Last Modif. 07/02/11 *" << endl;
+  cout << "*           Bio++ Sequence Manipulator, version 0.6              *" << endl;
+  cout << "* Author: J. Dutheil                        Last Modif. 21/12/11 *" << endl;
   cout << "******************************************************************" << endl;
   cout << endl;
   
@@ -197,15 +201,15 @@ int main(int args, char** argv)
     // +-------------+
     else if (cmdName == "Translate")
     {
-      if (!AlphabetTools::isNucleicAlphabet(sequences->getAlphabet()))
-        throw Exception("Error in translation: alphabet is not of type 'nucleic'.");
+      if (!AlphabetTools::isCodonAlphabet(sequences->getAlphabet()))
+        throw Exception("Error in translation: alphabet is not of type 'codon'.");
       GeneticCode* gc = NULL;
       string gcstr = ApplicationTools::getStringParameter("code", cmdArgs, "Standard");
-      gc = SequenceApplicationTools::getGeneticCode(dynamic_cast<const NucleicAlphabet *>(sequences->getAlphabet()), gcstr);
+      gc = SequenceApplicationTools::getGeneticCode(dynamic_cast<const CodonAlphabet*>(sequences->getAlphabet())->getNucleicAlphabet(), gcstr);
 
       OrderedSequenceContainer* sc = 0;
-      if (aligned) sc = new VectorSiteContainer(sequences->getAlphabet());
-      else         sc = reinterpret_cast<OrderedSequenceContainer*>(new VectorSequenceContainer(sequences->getAlphabet()));
+      if (aligned) sc = new VectorSiteContainer(&AlphabetTools::PROTEIN_ALPHABET);
+      else         sc = reinterpret_cast<OrderedSequenceContainer*>(new VectorSequenceContainer(&AlphabetTools::PROTEIN_ALPHABET));
       for (unsigned int i = 0; i < sequences->getNumberOfSequences(); i++)
       {
         Sequence* seq = gc->translate(sequences->getSequence(i));
@@ -223,9 +227,9 @@ int main(int args, char** argv)
       VectorSequenceContainer* sc = new VectorSequenceContainer(sequences->getAlphabet());
       for (unsigned int i = 0; i < sequences->getNumberOfSequences(); i++)
       {
-        Sequence* seq = SequenceTools::removeGaps(sequences->getSequence(i));
+        auto_ptr<Sequence> seq(sequences->getSequence(i).clone());
+        SequenceTools::removeGaps(*seq);
         sc->addSequence(*seq);
-        delete seq;
       }
       delete sequences;
       sequences = sc;
@@ -273,10 +277,40 @@ int main(int args, char** argv)
     // +--------------+
     else if (cmdName == "RemoveStops")
     {
-      SiteContainer* sites = dynamic_cast<SiteContainer *>(sequences);
+      SiteContainer* sites = dynamic_cast<SiteContainer*>(sequences);
       if (!sites)
       {
-        throw Exception("'RemoveStops' can only be used on alignment. You may consider using the 'CoerceToAlignment' command.");
+        VectorSequenceContainer* sc = new VectorSequenceContainer(sequences->getAlphabet());
+        for (unsigned int i = 0; i < sequences->getNumberOfSequences(); ++i)
+        {
+          auto_ptr<Sequence> seq(sequences->getSequence(i).clone());
+          SequenceTools::removeStops(*seq);
+          sc->addSequence(*seq);
+        }
+        delete sequences;
+        sequences = sc;
+      } else {
+        VectorSiteContainer* sc = new VectorSiteContainer(sequences->getAlphabet());
+        for (unsigned int i = 0; i < sequences->getNumberOfSequences(); ++i)
+        {
+          auto_ptr<Sequence> seq(sequences->getSequence(i).clone());
+          SequenceTools::replaceStopsWithGaps(*seq);
+          sc->addSequence(*seq);
+        }
+        delete sequences;
+        sequences = sc;
+      }
+    }
+
+    // +--------------+
+    // | Remove stops |
+    // +--------------+
+    else if (cmdName == "RemoveColumnsWithStop")
+    {
+      SiteContainer* sites = dynamic_cast<SiteContainer*>(sequences);
+      if (!sites)
+      {
+        throw Exception("'RemoveColumnsWithStop' can only be used on alignment. You may consider using the 'CoerceToAlignment' command.");
       }
 
       for (unsigned int i = sites->getNumberOfSites(); i > 0; i--)
@@ -297,7 +331,12 @@ int main(int args, char** argv)
       for (unsigned int i = 0; i < sequences->getNumberOfSequences(); i++)
       {
         BasicSequence seq = sequences->getSequence(i);
+        unsigned int len = seq.size();
         SequenceTools::getCDS(seq, false, true, true, false);
+        if (aligned) {
+          for (unsigned int c = seq.size(); c < len; ++c)
+            seq.addElement(seq.getAlphabet()->getGapCharacterCode());
+        }
         sc->addSequence(seq, false);
       }
       delete sequences;
@@ -355,7 +394,7 @@ int main(int args, char** argv)
         {
           map<int, double> freqs;
           SiteTools::getFrequencies(sites->getSite(i - 1), freqs);
-          if (freqs[-1] >= gapFreq) sites->deleteSite(i - 1);
+          if (freqs[-1] > gapFreq) sites->deleteSite(i - 1);
         }
       }
       else
@@ -388,6 +427,41 @@ int main(int args, char** argv)
       delete sequences;
       sequences = sc;
     }
+    // +------------------+
+    // | GetCodonPosition |
+    // +------------------+
+    else if (cmdName == "GetCodonPosition")
+    {
+      unsigned int pos = ApplicationTools::getParameter<unsigned int>("position", cmdArgs, 3);
+      OrderedSequenceContainer* sc = dynamic_cast<OrderedSequenceContainer*>(SequenceContainerTools::getCodonPosition(*sequences, pos - 1));
+      delete sequences;
+      if (aligned) {
+        sequences = new VectorSiteContainer(*sc);
+        delete sc;
+      } else {
+        sequences = sc;
+      }
+    }
+    // +-----------------+
+    // | FilterFromTree |
+    // +-----------------+
+    else if (cmdName == "FilterFromTree")
+    {
+      auto_ptr<Tree> tree(PhylogeneticsApplicationTools::getTree(cmdArgs, ""));
+      vector<string> names = tree->getLeavesNames();
+      OrderedSequenceContainer* reorderedSequences = 0;
+      if (aligned) {
+        reorderedSequences = new VectorSiteContainer(sequences->getAlphabet());
+      } else {
+        reorderedSequences = new VectorSequenceContainer(sequences->getAlphabet());
+      }
+      for (size_t i = 0; i < names.size(); ++i) {
+        reorderedSequences->addSequence(sequences->getSequence(names[i]), false);
+      }
+      delete sequences;
+      sequences = reorderedSequences;
+    }
+
     else throw Exception("Unknown action: " + cmdName);
   }
   
